{%- from "kerko/_facets.html.jinja2" import facet %}

{{- facet(
        spec.title,
        spec.missing_label,
        items,
        wrap_class='row',
        heading_tag='div',
        heading_class='col-md-4 col-xl-3 font-weight-bold break-word text-md-right text-print-left',
        body_class='col-md-8 col-xl-9',
        list_class_top='list-unstyled mb-0'
    )
}}
