{%- from "kerko/_facets.html.jinja2" import facet %}

{{- facet(
        spec.title,
        spec.missing_label,
        items,
        add_icon='',
        add_title=_("Search with the '{}' filter"),
        remove_icon='',
        icon_left='',
        icon_top='',
        icon_line_height='',
        wrap_class='row mb-3',
        heading_tag='div',
        heading_class='col-md-4 col-lg-3 font-weight-bold break-word text-md-right text-print-left',
        body_class='col-md-8 col-lg-9 break-word',
        list_class_top='list-unstyled mb-0'
    )
}}
