.list-no-bullets {
  list-style-type: none;
}
.no-decorate:hover,
.no-decorate:focus {
  text-decoration: none;
}
:hover > .parent-decorate,
:focus > .parent-decorate {
  text-decoration: underline;
}
.no-js .print-link {
  display: none;
}
.break-word {
  word-wrap: break-word; /* Older, non-standard name for overflow-wrap. */
  overflow-wrap: break-word;
}
.pre-line {
  white-space: pre-line;
}
.breadbox > .card-body > * {
  margin-top: 1rem;
}
.breadbox > .card-body > *:first-child {
  margin-top: 0;
}
@media print {
  .row {
    /*
      Fix for truncated pages in Firefox (issue with display: flex).
      Ref: https://bugzilla.mozilla.org/show_bug.cgi?id=939897
      Breaks column layouts, but we can live with that.
    */
    display: block !important;
  }
  .tab-content > .tab-pane {
    display: block;
    opacity: 1;
  }
  .text-md-right.text-print-left {
    text-align: left !important;
  }
  .breadbox a,
  .search-item a,
  .item-content a {
    text-decoration: none;
  }
}
.navbar-brand {
  white-space: normal;
}
.dropdown-width-300 {
  width: 300px;
  max-width: 90vw;
}
.dropdown-item {
  white-space: normal;
}
.btn-expand[aria-expanded="true"] {
  display: none;
}
.btn-expand[aria-expanded="true"] + .btn-collapse {
  display: inline;
}
.btn-expand[aria-expanded="false"] {
  display: inline;
}
.btn-expand[aria-expanded="false"] + .btn-collapse {
  display: none;
}
.continued-hint {
  -webkit-mask-image: linear-gradient(to top, transparent, black 2.75rem);
  mask-image: linear-gradient(to top, transparent, black 2.75rem);
}
main {
  min-height: calc(100vh - 7rem);
  padding-top: 1rem;
  padding-bottom: 3rem;
}
.btn-toggle[active] > .btn-toggle-icon-off {
  display: none;
}
.btn-toggle:not([active]) > .btn-toggle-icon-on {
  display: none;
}
.d-none-important {
  display: none !important;
}
