{% for item in config.kerko_composer.link_groups.navbar.get_ordered_links() %}
    <li class="nav-item {% if item.is_active(request) %}active{% endif %}">
        <a class="nav-link" href="{{ item.url }}" {% if item.new_window %}target="_blank" rel="noopener"{% endif %}>{{ item.text }}</a>
    </li>
{% endfor %}
